#include ".\..\General\General.h"
#include "Countdown.h"

countdown_t volatile count;

#ifdef __GNUC__
#    define inline __attribute__((always_inline))
#endif // GCC

static inline void down (uint8_t, char volatile *);

void down (uint8_t i, char volatile * cd)
{
    // Komponenten (i Stck) von hinten nach vorne durchlaufen
    // Zeiger cd steht anfangs hinter der letzten Komponente
    do
    {
        // Zeiger erniegrigen, Wert lesen und um 1 vermindern
        char val = *(--cd) - 1;

        // Falls der Wert nicht abgelaufen ist wird er (vermindert)
        // zurckgeschrieben, ansonsten bleibt er bei 0.
        if (val != -1)
            *cd = val;
    } while (--i);
}

#define ITEM_COUNT(A,B) sizeof (A) / sizeof (B)

#define DOWN(x)             \
    do {                    \
        if (!sizeof (x))    \
            return;         \
        down (ITEM_COUNT(x, char), (char*) & x + ITEM_COUNT(x, char)); \
    } while (0)

// Alle 10 Millisekunden aufrufen
void job_countdown (void)
{
    // Die 10ms-Werte von 'count' runterzhlen (8-Bit Werte)
    DOWN   (count.ms10);

    // .ms10.timer_1s abgelaufen?
    if (count.ms10.timer_1s)
        // Nein
        return;

    // Ja. Eine neue Sekunde beginnt: Sekunden-Zhler neu aufziehen
    count.ms10.timer_1s = 100;

    // Die Sekunden-Werte von 'count' runterzhlen
    DOWN (count.sec);

    // Falls bei den Minuten nicht zu tun ist sind wir fertig.
    if (!sizeof (count.min))
        return;

    // Minuten-Zhler abgelaufen?
    if (count.sec.timer_1m)
        // Nein
        return;

    // Minuten-Zhler neu aufziehen
    count.sec.timer_1m = 60;

    // Die Minuten-Werte von 'count' runterzhlen
    DOWN (count.min);
}

void Countdown_Init (void)
{
    // PoutputCompare fr gewnschte Timer1 IRQ-Rate
    OCR1A = (unsigned short) ((unsigned long) F_CPU / IRQS_PER_SECOND-1);

    // Timer1 ist Zhler: Clear Timer on Compare Match (CTC, Mode #4)
    // Timer1 luft @ F_CPU: prescale = 1
    TCCR1A = 0;
    TCCR1B = (1 << WGM12) | (1 << CS10);

    // OutputCompare-Interrupt A fr Timer1
    TIMSK = (1 << OCIE1A);
}

ISR (TIMER1_COMPA_vect)
{
    static uint8_t interrupt_num_10ms;

    uint8_t irq_num = interrupt_num_10ms;

    // interrupt_num_10ms erhhen und mit Maximalwert vergleichen
    if (++irq_num == IRQS_PER_10MS)
    {
        // 10 Millisekunden sind vorbei
        // interrupt_num_10ms zurcksetzen
        irq_num = 0;
    }

    if (irq_num == 0)       job_countdown();
    //if (irq_num == 1)     noch was erledigen alle 10ms, aber bitte nicht alles auf einmal ;-)
    //if (irq_num == 2)     ...

    interrupt_num_10ms = irq_num;
}
