#ifndef UART_EXT_H
#define UART_EXT_H
/*@{*/

#if (__GNUC__ * 100 + __GNUC_MINOR__) < 303
#error "This library requires AVR-GCC 3.3 or later, update to newer AVR-GCC compiler !"
#endif

#include ".\..\UART\uart.h"

/**
 * @brief    Put integer to ring buffer for transmitting via UART.
 *
 * The integer is converted to a string which is buffered by the uart 
 * library in a circular buffer and one character at a time is transmitted 
 * to the UART using interrupts.
 *
 * @param    value to transfer
 * @return   none
 * @see      uart_puts_p
 */
extern void uart_puti( int i );

/**
 * @brief    Put nibble as hex to ring buffer for transmit via UART.
 *
 * The lower nibble of the parameter is converted to corresponding
 * hex-char and put in a circular buffer and one character at a time 
 * is transmitted to the UART using interrupts.
 *
 * @param    value to transfer (byte, only lower nibble converted)
 * @return   none
 * @see      uart_putc
 */
extern void uart_puthex_nibble(const unsigned char b);

/**
 * @brief    Put byte as hex to ring buffer for transmit via UART.
 *
 * The upper and lower nibble of the parameter are converted to 
 * corresponding hex-chars and put in a circular buffer and one 
 * character at a time is transmitted to the UART using interrupts.
 *
 * @param    value to transfer
 * @return   none
 * @see      uart_puthex_nibble
 */
extern void uart_puthex_byte(const unsigned char b);

/*@}*/
#endif // UART_EXT_H 

